sezaru

sezaru

So, I’m importing millions of rows in my system to a resource that has some fields (around 7 fields) that are embedded resources.

I’m inserting the fields using bulk_create with chunks of 10_000 rows.

I noticed that this operation was kinda slow, it takes around 36 seconds to insert the 10_000 rows.

I suspected that the issue could be because of the embedded resources, so I removed them and replaced with fields directly in the main resource.

After that, the same bulk_create call takes 6 seconds to do the same work.

So, it seems that a embedded resource creates a big overhead when inserting a bunch of data.

Zack, do you think there is something that can be done to improve that, or that overhead is to be expected?

Showing Posts 1 to 10

zachdaniel

zachdaniel

Creator of Ash

This definitely strikes me as something that should be fixable. It’s possible that the overhead is in the data layer itself, but I think that is probably unlikely. I would need to investigate further, and may not have time to do that this week. You can confirm that the performance issue is Ash by orchestrating a similar operation with bare ecto if you have some time :slight_smile:

sezaru

sezaru OP

I will try that when I have some free time and report back to you :smiley:

zachdaniel

zachdaniel

Creator of Ash

So, someone else reported a similar issue with casting embedded resources. I hope to have some time to look into it later this week.

barnabasJ

barnabasJ

Ash Core Team

I had a similar issue. I had a calculation in which I made an API request and used a resource with Ash.DataLayer.Simple to convert the data in the response into a list of Ash Resources to have nice Graphql types.

The Resource in question also had a couple of embedded resources and also nested embedded resources. This was also really slow.

The only way I was able to get it to a reasonable timeframe was to do all the conversions myself and create the structs for the embedded resources myself before passing the data to API.bulk_create with the assume_casted?: true option.

zachdaniel

zachdaniel

Creator of Ash

Alright, beginning to look into this issue. There is definitely a significant slow down on embedded resource casting

Name             ips        average  deviation         median         99th %
maps         23.69 K      0.0422 ms    ±56.44%      0.0393 ms       0.107 ms
embeds        0.25 K        4.06 ms     ±4.56%        4.03 ms        4.66 ms
zachdaniel

zachdaniel

Creator of Ash

Okay so I’ve made some performance improvements. They are still much slower than raw maps, but if you guys could try it out and report back, that would be great. Performance improvements are in main. If you update ash to main, make sure to update ash_postgres and ash_graphql also.

barnabasJ

barnabasJ

Ash Core Team

After getting everything working with ash/main again, I can say that our calculation is now about 3x faster than before.

zachdaniel

zachdaniel

Creator of Ash

Thats progress! Are you still having to manually create the embedded resources?

barnabasJ

barnabasJ

Ash Core Team

I do for now, as I’m still using assume_casted?: true for the bulk_create. Casting ist still taking up too much time in this specific instance.

I just finished upgrading, so I haven’t done any extensive testing yet but here is what I have right now.

I have these 2 reads, the first one is what I tried first but wasn’t fast enough.

I did a little testing with benchee, running the actions with the same list of 7 options each, so I don’t think bulk_create should a lot of optimization there.

Reading with doing the casting:

    read :read_options do
      argument :options, {:array, :map} do
        allow_nil? false
      end

      prepare before_action(fn query ->
                Simple.set_data(
                  query,
                  Ash.Query.get_argument(query, :options)
                  |> Enum.map(&__MODULE__.from_option!/1)
                )
              end)
    end

Takes around 10ms

vs

Preparing the input myself + doing this read action

      prepare before_action(fn query ->
                Simple.set_data(
                  query,
                  Ash.Query.get_argument(query, :availabilities)
                  |> Api.bulk_create(
                    __MODULE__,
                    :create,
                    assume_casted?: true,
                    return_records?: true,
                    return_errors?: true,
                    authorize?: false,
                    max_concurrency: 10
                  )
                  |> then(fn %Ash.BulkResult{records: records, errors: errors} ->
                    records
                  end)
                )
              end)
    end

Takes around 1ms

The thing with assume_casted?: true is that the embedded resources already need to be the real Ash.Resource structs. It might be nice to have an option where it would turn the maps into the structs but wouldn’t cast the values for it.

zachdaniel

zachdaniel

Creator of Ash

Interesting…I definitely want to track down the main slow down. I think there is likely just some unnecessary work being done there.

Where Next? Top

Trending in Questions Top

stjefim
Hello! Suppose you are building workflow (order / task / payment) processing system with the following requirements: Each workflow con...
New
water
I’m aware of 2 such projects, are there others? git: GitHub - team-alembic/realworld: A fullstack Phoenix LiveView application with back...
New
itzmidinesh
Hi all, I’m working on a Phoenix project where I need to make external HTTP requests and I’m trying to choose between Req and HTTPoison....
New
pragdave
How do I tell Hologram not to escape HTML when substituting into templates? {Textile.render(@product.long_description)} escapes the HTM...
New
rlipscombe
I’d like to create a file with 600 permissions (rw-------), but there’s a race condition if I create it and then change the permissions. ...
New
leoschweizer
Hi, we run different LLM models across a cluster, with each node loading a configurable subset. Model-specific queues let nodes consume ...
New
rlipscombe
When I use Mix.install with a git dependency, using git@github.com:... how do I change the SSH identity used? I’ve got two Github accoun...
New

Other Trending Topics Top

dominicletz
Hey there, I haven’t seen a list of useful Elixir specific skills for the popular AI tools yet so I’m starting one here with my first. ...
New
raa-ziq
Hi everyone :waving_hand: I wanted to share something I’ve been building — NowBlind, a random chat app designed to help people discover ...
New
mudasobwa
I fully migrated to my own harness from Anthropic/Gemini and I think it’s time to share it. Welcome DSH, the DeepSeek Harness, fully writ...
New
KristerV
Hey. Is there anyone here who creates agents in their apps? Not talking about using agents, but creating them. I’m finding it pretty diff...
New
OndrejValenta
(I just needed to vent somewhere and LinkedIn is full of hope, or hype, I’m not sure which exactly) AI dream has many faces, but general...
New
kip
Localize is the next generation localisation library for Elixir. Think of it as ex_cldr version 3.0. The first version will be released ...
New

We're in Beta

About us Mission Statement

Options

Thread Display Mode




Thread Preview

Skip Thread Previews